#!/usr/bin/perl -w

# differs from calc_ident1.pl only that input file motif is read from a parameter

# INPUT: 	xxx_short_final.tbl hits to be treated (without redundant sequences but with new concat sequences, no autohits, no reciproc hits and no multiple hits)
#		xxx_nr.fas	fasta file with all non-red sequences including original and concatenated ones
#		(output files of group_seq_var.pl)
#		xxx_nr.seq (output of extract_perfect_microsat_with_hach.pl run on nr file with appropriate ms length settings)

#only 1 series of files in the input folder at a time 

# OUTPUT: 	xxx_cl.txt 
#			[0] seq1
#			[1] seq2
#			[2] first pos seq1
#			[3] last pos seq1
#			[4] first pos seq2
#			[5] last pos seq2
#			[6] positions (in alignement) of substitution (separated by spaces)
#			[7] positions (in alignement) of qap in seq1 (separated by spaces)
#			[8] positions (in alignement) of gap_in seq2 (separated by spaces)
#			[9] prop of mismatch (indel+subst) in whole seq
#			[10] prop of mismatch (indel+subst) in flankig region only
#				(my $perc_all = (scalar@q_gap + scalar@s_gap + scalar@subst)/$align_l;)
#				(my $perc_fl =  (scalar@q_gap_fl + scalar@s_gap_fl + scalar@subst_fl)/($cut_pos[1] - $cut_pos[0] + 1 + scalar@q_gap_fl + scalar@s_gap_fl);)

#		xxx_gr.txt (blast hit lines where overhang is to long)



  use warnings;
  use strict;
  use subprogramEM;

my $folder = '';
my $outfolder = '';
my $syst = '';
my $e = 0;
my $blast_path = '';
my $clustal_path = '';
my $clust_ver = 0;
my $overhang = 0;
# enlarge ms positions my $ms_plus pb at each side of the ms (border ms, flank incertain, 
#if indel in ms at the edge, by adding 1 bp indel automatically counted as part of the ms) 
my $ms_plus = 0;
my $file_mot = '';
 


if ( scalar@ARGV == 10)
{
	$syst = $ARGV[0];
	$folder = $ARGV[1];
	$outfolder = $ARGV[2];
	$blast_path = $ARGV[3];
	$clustal_path = $ARGV[4];
	$clust_ver = $ARGV[5];
	$e = $ARGV[6];
	$overhang = $ARGV[7];
	$ms_plus = $ARGV[8];
	$file_mot = $ARGV[9];
}
else
{
	$folder = 'datatemp1';
	$outfolder = 'temp1';
	#$syst = 'linux';
	$syst = 'win';
	$e = '1E-40';
	#$blast_path = '/usr/local/bin/blast-2.2.13/bin/';
	#$blast_path = '/home/user0/bin/blast-2.2.19/bin/';
	$blast_path = 'c:/BLAST2_2_18/bin/';
	#$clustal_path = '/home/user0/bin/clustalw-2.0.10/';
	$clustal_path = 'c:/CLUSTALW/clustalw1.83.XP/';
	$clust_ver = 1;
	$overhang = 20;
# enlarge ms positions my $ms_plus pb at each side of the ms (border ms, flank incertain, 
#if indel in ms at the edge, by adding 1 bp indel automatically counted as part of the ms) 
	$ms_plus = 1;
	$file_mot = '_nr.fas';
}



  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

my $temp_file = $outfolder.'/temp.txt';
my $blast_file = '';
my $fas_nr_file = '';
my $seq_file = '';
my $outfile_gr = '';
my $outfile_cl = '';
my $clust_inp_file = $outfolder.'/clust';
my $script_file = $outfolder.'/script_file';
my $clust_out = $clust_inp_file.'.aln';

make_script_file($script_file, $clust_inp_file);


# define filenames
foreach my $filename (@filenames)
{
	if ($filename =~ /$file_mot$/)
	{
		$outfile_gr = $filename;
		$outfile_gr =~ s/_nr\.fas/_gr\.txt/;
		$outfile_gr = $outfolder.'/'.$outfile_gr;
		$outfile_cl = $outfile_gr;
		$outfile_cl =~ s/_gr\.txt/_cl\.txt/;
	$filename = $folder.'/'.$filename;
		$fas_nr_file = $filename;
		$seq_file = $filename;
		$seq_file =~ s/\.fas/\.seq/;
		$blast_file = $filename;
		$blast_file =~ s/_nr\.fas/_short_final\.tbl/;


my %seq = read_hach_from_fasta_code_wo_space($fas_nr_file);
my %seq_l = length_sequences(\%seq);
my %ms_info = open_file_and_hach($seq_file, '0'); 
my $prev_query = '';
my %ms_pos = ();
my $q_beg = '';
my $align_l = '';
#print_hachage(\%ms_inf);

open_file_out($outfile_gr, 'OUT');
open_file_out($outfile_cl, 'SIM');

open_file_in($blast_file, 'IN');

while(my $line = <IN>)
{
# avoid occasional empty lines at the end of the file
	if ($line eq "\n")
	{next;}
	my @line = split(' ', $line);
#for each line extract if overhang beyond the blast aligned zone is short
# cut sequences to aling and put query and sbj into a fasta file for clustal
# if overhang is long put seq pair in grouped file
	my $cut_pos = select_cover_overlap_overhang(\@line, \%seq, \%seq_l, $overhang, $clust_inp_file);
	if ($cut_pos eq 'gr')
	{
		print OUT "$line";
	}
# if overhang short call clustal
	else
	{
	    if ($clust_ver == 2)
	    {
		call_clustalw2_path($script_file, $clustal_path, $outfolder);
	    }
	    else
	    {
		call_clustalw_path($script_file, $clustal_path, $outfolder);		
	    }
		my $q_align = '';
		my $s_align = '';
		extract_aligned_seq($clust_out, $line[0], $line[1], \$q_align, \$s_align);
		my @subst = ();
		my @q_gap = ();
		my @s_gap = ();
#                print "$line[0]-$line[1] ";
#		print "$q_align************\n";
#		print "$s_align************\n";

		$align_l = length $q_align;

		count_mismatches($q_align, $s_align, \@subst, \@q_gap, \@s_gap);
print SIM "$line[0];$line[1];$cut_pos$align_l;@subst;@q_gap;@s_gap;";
#-------------------------------------------------------------
# Transfrorm positions in the alignement into pos in the query seq.
# pos of gaps in the query are also transformed. 
#For each gap it gives that number of bases in the query beore the given gap, Thus, if
#gap is longer that 1, the sale query pos is printed as many times as consequetive gaps are
# if query sequence was cut at the begining change positions into positios in the original query seq
		
		my @cut_pos = split(';', $cut_pos);

		transform_pos_in_align_into_pos_in_seq_1(\@q_gap, \@subst, $cut_pos[0]);
		transform_pos_in_align_into_pos_in_seq_1(\@q_gap, \@s_gap, $cut_pos[0]);
		transform_pos_in_align_into_pos_in_seq_1(\@q_gap, \@q_gap, $cut_pos[0]);

		unless ($line[0] eq $prev_query)
		{
#			print "$ms_info{$line[0]}\n";
			%ms_pos = make_ms_pos_hach($ms_info{$line[0]}, $ms_plus);
#			print_hachage(\%ms_pos);
		}
		$prev_query = $line[0];
		my @subst_fl = ();
		my @q_gap_fl = ();
		my @s_gap_fl = ();
		sort_mismatches_into_ms_or_flank(\%ms_pos, \@q_gap, \@s_gap, \@subst, \@q_gap_fl, \@s_gap_fl, \@subst_fl);

		my $perc_all = (scalar@q_gap + scalar@s_gap + scalar@subst)/$align_l;
		my $perc_fl =  (scalar@q_gap_fl + scalar@s_gap_fl + scalar@subst_fl)/($cut_pos[1] - $cut_pos[0] + 1 + scalar@q_gap_fl + scalar@s_gap_fl);
		print SIM "$perc_all;$perc_fl;\n";
# end else
	}

# end while
}

close (IN);
close (OUT);
close (SIM);
if (0)
{
# delete non_essential outfiles
my $motdel = '';
if ($syst eq 'win')
{
	$motdel = 'script_file';
	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clust';
#	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clust.aln';
	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clust.dnd';
	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clustout';
	delete_files_motif_win($outfolder, $motdel);
}
elsif ($syst eq 'linux')
{
	$motdel = 'script_file';
	delete_files_motif($outfolder, $motdel);
	$motdel = 'clust';
#	delete_files_motif($outfolder, $motdel);
	$motdel = 'clust.aln';
	delete_files_motif($outfolder, $motdel);
	$motdel = 'clust.dnd';
	delete_files_motif($outfolder, $motdel);
	$motdel = 'clustout';
	delete_files_motif($outfolder, $motdel);
}
}

#read all file in the outfolder and deletes then is unnecessary
unless ( opendir(FOLDER, $outfolder) )
{
      print "Cannot access to folder $folder\n";
      exit;
}
my @filenames_del = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);
foreach my $file (@filenames_del)
{
    if ($file eq $script_file or $file eq 'clustout' or $file =~ /clust\./)
    {
	delete_win_linux($file, $outfolder, $syst);
    }
}

# end if file_mot
	}
# end foreach file
}

exit;

##############################################################
##############################################################
##############################################################
##############################################################
sub select_cover_overlap_overhang
{

my ($line, $seq, $seq_l, $overhang, $file) =@_;
my $cover = '';
my $seqq = '';
my $seqs = '';
my $q_beg = '';
my $q_end = '';
my $s_beg = '';
my $s_end = '';
my $brin = '';
my $cut_pos = '';
open_file_out($file, 'CLUST');


# %$seq_l is the hach with the length of the sequences

# $cover = 1 cover, query is shorter than sbj; 
#$cover = 2 cover, sbj is shorter than query 
#$cover = 3 overlap at the end of the query, (beg of sbj if +, end of sbj if -)
#$cover = 4 overlap at the beg of the query, (end of sbj if +, beg of sbj if -)
#$cover = 5 overhang-cover query is shorter than $subj
#$cover = 6 overhang-cover sbj is shorter than $query
#$cover = 7 overhang-overlap at the end of the query (beg of sbj if +, end of sbj if -)
#$cover = 8 overhang-overlap at the beg of the query (end of sbj if +, beg of sbj if -)
#$cover = 9 non_aligned sequence is longer then overhang

#[0] query [1] sbj [2] %id;[3]HSP lenght;[4]mismath?;[5]gap?;[6]query_first;[7]query_last;[8]sbj first;[9]sbj_last;[10]e;[11]score;

# brin+
	    if ($$line[8] < $$line[9])
	    {
		$brin = 'p';
#overhang
#query is shorter than $subj cover complete brin+
		if ($$line[6] < $overhang and ($$seq_l{$$line[0]} - $$line[7]) < $overhang  )
		{
			$cover = 5;
			$q_beg = 1;
			$q_end = $$seq_l{$$line[0]};
			$seqq = $$seq{$$line[0]};
			$s_beg = $$line[8] - $$line[6] +1;
			if ($s_beg < 1)
			{
				$s_beg = 1;
			}
                  $s_end = $$seq_l{$$line[0]}-$$line[7] + $$line[9];
			if ($s_end > $$seq_l{$$line[1]})
			{
				$s_end = $$seq_l{$$line[1]};
			}
			$seqs = substr($$seq{$$line[1]}, $s_beg-1, $s_end - $s_beg +1);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';

		}
#overhang
#sbj is shorter than query cover complete brin+
		elsif ($$line[8] < $overhang and ($$seq_l{$$line[1]} - $$line[9]) < $overhang )
		{
			$cover = 6;
			$s_beg = 1;
			$s_end = $$seq_l{$$line[1]};
			$seqs = $$seq{$$line[1]};
			$q_beg = $$line[6] - $$line[8] +1;
			if ($q_beg < 1)
			{
				$q_beg = 1;
			}
                  $q_end = $$seq_l{$$line[1]}-$$line[9] + $$line[7];
			if ($q_end > $$seq_l{$$line[0]})
			{
				$q_end = $$seq_l{$$line[0]};
			}
			$seqq = substr($$seq{$$line[0]}, $q_beg-1, $q_end - $q_beg +1);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';

		}
#overhang
# overlap at the end of the query, beg of sbj
		elsif ( ($$seq_l{$$line[0]}-$$line[7]) < $overhang and $$line[8] < $overhang)
		{
			$cover = 7;
			$q_beg = $$line[6] - $$line[8] +1;
			if ($q_beg < 1)
			{
				$q_beg = 1;
			}
			$q_end = $$seq_l{$$line[0]};
			$s_beg = 1;
                  $s_end = $$seq_l{$$line[0]}-$$line[7] + $$line[9];
			if ($s_end > $$seq_l{$$line[1]})
			{
				$s_end = $$seq_l{$$line[1]};
			}
			$seqq = substr($$seq{$$line[0]}, $q_beg-1, $q_end - $q_beg +1);
			$seqs = substr($$seq{$$line[1]}, $s_beg-1, $s_end - $s_beg +1);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';

		}
#overhang
# overlap at the end of the sbj, beg of query
		elsif ($$line[6] <$overhang and $$seq_l{$$line[1]} - $$line[9] < $overhang)
		{
			$cover = 8;
			$s_beg = $$line[8] - $$line[6] +1;
			if ($s_beg < 1)
			{
				$s_beg = 1;
			}
			$s_end = $$seq_l{$$line[1]};
			$q_beg = 1;
                  $q_end = $$seq_l{$$line[1]}-$$line[9] + $$line[7];
			if ($q_end > $$seq_l{$$line[0]})
			{
				$q_end = $$seq_l{$$line[0]};
			}
			$seqq = substr($$seq{$$line[0]}, $q_beg-1, $q_end - $q_beg +1);
			$seqs = substr($$seq{$$line[1]}, $s_beg-1, $s_end - $s_beg +1);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';

		}
		else
		{
			$cut_pos = 'gr';
		}

# end brin +
	    }
#brin-
	    else
	    {
                $brin = 'm';
#overhang
#query is shorter than $subj cover complete brin-
		if ($$line[6] < $overhang and ($$seq_l{$$line[0]} - $$line[7]) < $overhang)
		{
			$cover = 5;
			$q_beg = 1;
			$q_end = $$seq_l{$$line[0]};
			$seqq = $$seq{$$line[0]};
			$s_end = $$line[8] + $$line[6] -1;
			if ($s_end > $$seq_l{$$line[1]})
			{
				$s_end = $$seq_l{$$line[1]};
			}
                  $s_beg = $$line[9] - ($$seq_l{$$line[0]}-$$line[7]);
			if ($s_beg < 1)
			{
				$s_beg = 1;
			}
			$seqs = substr($$seq{$$line[1]}, $s_beg-1, $s_end - $s_beg +1);
			$seqs = reverse_complement($seqs);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';

		}
#overhang
#sbj is shorter than query cover complete brin-
		elsif (($$seq_l{$$line[1]} - $$line[8]) < $overhang  and $$line[9] < $overhang)
		{
			$cover = 6;
			$s_beg = 1;
			$s_end = $$seq_l{$$line[1]};
			$seqs = $$seq{$$line[1]};
			$seqs = reverse_complement($seqs);			
			$q_beg = $$line[6] - ($$seq_l{$$line[1]} - $$line[8]);
			if ($q_beg < 1)
			{
				$q_beg = 1;
			}
                  $q_end = $$line[9] -1 + $$line[7];
			if ($q_end > $$seq_l{$$line[0]})
			{
				$q_end = $$seq_l{$$line[0]};
			}
			$seqq = substr($$seq{$$line[0]}, $q_beg-1, $q_end - $q_beg +1);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';

		}
#overhang
# overlap at the end of the query, end of sbj
		elsif (($$seq_l{$$line[0]}-$$line[7]) < $overhang  and ($$seq_l{$$line[1]}-$$line[8])<$overhang )
		{
			$cover = 7;
			$q_beg = $$line[6] - ($$seq_l{$$line[1]} - $$line[8]);
			if ($q_beg < 1)
			{
				$q_beg = 1;
			}
			$q_end = $$seq_l{$$line[0]};
			$s_end = $$seq_l{$$line[1]};
                  $s_beg = $$line[9] - ($$seq_l{$$line[0]}-$$line[7]);
			if ($s_beg < 1)
			{
				$s_beg = 1;
			}
			$seqq = substr($$seq{$$line[0]}, $q_beg-1, $q_end - $q_beg +1);
			$seqs = substr($$seq{$$line[1]}, $s_beg-1, $s_end - $s_beg +1);
			$seqs = reverse_complement($seqs);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';

		}
#overhang
# overlap at the beg of the query, beg of sbj
		elsif ($$line[6] <$overhang and $$line[9] < $overhang)
		{
			$cover = 8;
			$q_beg = 1;
			$s_beg = 1;
                  $q_end = $$line[9] -1 + $$line[7];
			if ($q_end > $$seq_l{$$line[0]})
			{
				$q_end = $$seq_l{$$line[0]};
			}
			$s_end = $$line[8] + $$line[6] -1;
			if ($s_end > $$seq_l{$$line[1]})
			{
				$s_end = $$seq_l{$$line[1]};
			}
			$seqq = substr($$seq{$$line[0]}, $q_beg-1, $q_end - $q_beg +1);
			$seqs = substr($$seq{$$line[1]}, $s_beg-1, $s_end - $s_beg +1);
			$seqs = reverse_complement($seqs);
			print CLUST ">$$line[0]\n$seqq\n";
			print CLUST ">$$line[1]\n$seqs\n";
			$cut_pos = $q_beg.';'.$q_end.';'.$s_beg.';'.$s_end.';';
		}
		else
		{
			$cut_pos = 'gr';
		}


#end of else
	    }

	    
return $cut_pos;


}
    
##############################################################

sub make_script_file
{
my ($sript_file, $clust_inp_file) = @_;

open_file_out($sript_file, 'SC');

print SC "1\n$clust_inp_file\n2\n1\n\n\nX\nX\nX\n";

close (SC);
}

################################

sub extract_aligned_seq
{
my ($clust_out, $query, $sbj, $q_align, $s_align) = @_;
open_file_in($clust_out, 'CLI');
 my @data = <CLI>;
close (CLI);

foreach my $line (@data)
{
	if ($line =~ /^$query\s+/ )
	{
		$line =~ s/^$query\s+//;
		chomp $line;
		$$q_align .= $line;
	}
	elsif ($line =~ /^$sbj\s+/ )
	{
		$line =~ s/^$sbj\s+//;
		chomp $line;
		$$s_align .= $line;
	}
}
}


####################################################

sub count_mismatches
{
my ($q_seq, $s_seq, $subst, $query_gap, $subj_gap) = @_;
my @query = split('', $q_seq);
my @subj = split('', $s_seq);

for ( my $i = 0; $i < scalar @query; ++$i)
{
#print "";
	unless ($query[$i] eq $subj[$i])
	{
		if ($query[$i] eq '-')
		{
			push(@$query_gap, $i+1);
		}
		elsif ($subj[$i] eq '-')
		{
			push(@$subj_gap, $i+1);
		}
		elsif ($query[$i] eq 'N' or $subj[$i] eq 'N')
		{
			next;
		}
		else
		{
			push(@$subst, $i+1);
		}
	}
}
}
#################################################

sub transform_pos_in_align_into_pos_in_seq
{
my ($gap_pos, $subst) = @_;
my $last_s = 0;

push(@$gap_pos, 1000000000);

for(my $i = 0; $i < scalar@$gap_pos; ++$i)
{
        for(my $s = $last_s; ($s < scalar @$subst) and ($$subst[$s] < $$gap_pos[$i]); ++$s)
	{
		$$subst[$s] = $$subst[$s] - $i;
                $last_s = $s +1;
	}
}

pop@$gap_pos;

}

#################################################

sub transform_pos_in_align_into_pos_in_seq_1
{
my ($gap_pos, $subst, $q_beg) = @_;
my $last_s = 0;
my $k = $q_beg -1;

push(@$gap_pos, 1000000000);

for(my $i = 0; $i < scalar@$gap_pos; ++$i)
{
        for(my $s = $last_s; ($s < scalar @$subst) and ($$subst[$s] < $$gap_pos[$i]); ++$s)
	{
		$$subst[$s] = $$subst[$s] - $i +$k;
                $last_s = $s +1;
	}
}

pop@$gap_pos;

}

##########################################################
sub transform_pos_in_cut_seq_into_pos_in_orig_seq
{
my ($q_beg, $subst) = @_;
my $k = $q_beg - 1;
for (my $i = 0; $i < scalar @$subst; ++$i)
{
	$$subst[$i] = $$subst[$i] + $k;
}	
}
####################################################

sub make_ms_pos_hach
{
my ($info, $ms_plus) = @_;
my %ms_pos = ();

# enlarge ms positions my $ms_plus pb at each side of the ms (border ms, flank incertain, 
#if indel in ms at the edge, by adding 1 bp indel automatically counted as part of the ms) 

# info line:
#seq_code;number_of_ms;seq_length;ms_motif;ms_first_pos;ms_last_pos;number_of_rep

my @info = split(';', $info);
for (my $i= 3; $i < scalar @info; $i = $i + 4)
{
	for (my $j = $info[$i]-$ms_plus; $j < $info[$i+1]+1+$ms_plus; ++$j)
	{
		$ms_pos{$j} = '';
	}
}

#print_hachage(\%ms_pos);
return %ms_pos 
}

#################################################
sub sort_mismatches_into_ms_or_flank
{
my($ms_pos, $q_gap, $s_gap, $subst, $q_gap_fl, $s_gap_fl, $subst_fl) = @_;

foreach my $pos (@$q_gap)
{
	unless (exists $$ms_pos{$pos})
	{
		push (@$q_gap_fl, $pos);
	}
}

foreach my $pos (@$s_gap)
{
	unless (exists $$ms_pos{$pos})
	{
		push (@$s_gap_fl, $pos);
	}
}

foreach my $pos (@$subst)
{
	unless (exists $$ms_pos{$pos})
	{
		push (@$subst_fl, $pos);
	}
}


}
####################################################
####################################################
#################################################

################################################

sub gap_positions
{
my ($q_seq) = @_;
my @pos = ();

my @seq = split('', $q_seq);
for (my $i = 0; $i < scalar @seq; ++$i)
{
	if ($seq[$i] eq '-')
	{
		push(@pos, $i+1);
	}
}

return @pos;
}
