#!/usr/bin/perl -w

# Nom du programme: pool_int_comp_ms.pl 

#INPUT: xxx_select.seq (output file of exatract_perfect_microsat.pl or find and mask_ms.pl)
# [0] SEQ_CODE
# [1] NUMBER OF MS
# [2] SEQ_LENGTH
# [3] MS MOTIF
# [4] MS FIRTS POS
# [5] MS LAST POS
# [6] NUMBER OF REP
# [3-6] REPEATED FOR ALL MS
#       xxx_select.fas 
#       ONLY 2 FILES IN FOLDER

#Screens all seq. and pools MS if 
#	-dist between 2 ms (including nano_sat) is shorter the user def limit de MS positions are merged, 
#	and pool is written into the motif
#	the number of repetition is the highest among the perfect MS in the pooled MS
#	selects only pooled ms as target if the flankig region conditions are met
# 	writes a modified .seq file



#OUTPUT:
# xxx_pool.seq modified .seq file where near ms are pooled




  use warnings;
  use strict;
  use subprogramEM;


my $folder = '';
my $outfolder = '';
# max number of bp between 2 me for pooling
my $diff = 2;
my $file_mot = '';
my $mono_max = '';
my $non_mono_max = ''; 


if (scalar@ARGV == 6)
{
	$folder = $ARGV[0];
	$outfolder = $ARGV[1];
	$diff = $ARGV[2];
	$file_mot = $ARGV[3];
	$mono_max = $ARGV[4];
	$non_mono_max = $ARGV[5]; 
}
else
{
	$folder = 'datatemp';
	$outfolder = 'temp';
	$diff = 2;
	$file_mot = '\.seq';
	$mono_max = 5;
	$non_mono_max = 5; 
}

  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

my $seq_file = '';
foreach my $file (@filenames)
{
if ($file =~ /$file_mot/ and not($file =~ /_compound\.seq/))
{
	my $out_seq = $file;
	$out_seq =~ s/\./_compound\./;
	$out_seq = $outfolder.'/'.$out_seq;

    $seq_file = $folder.'/'.$file;

open_file_in($seq_file, 'SEQ');
open_file_out($out_seq, 'OUT');

	my $title = <SEQ>;
	while (my $line = <SEQ>)
	{
		my @line = split(';', $line);
# to avoid last empty element after the last ;
   		pop@line;

		print OUT "$line[0];$line[1];$line[2];";
		my $beg = 0;
		my $end = 0;
		my $rep = 0;
		$line = $line[0].';'.$line[1].';'.$line[2].';';
		for (my $i = 6; $i< scalar@line; $i= $i+4)
		{
			if (length $line[$i-3] == 1 and $line[$i] > $mono_max -1)
			{
				$line .= $line[$i-3].';'.$line[$i-2].';'.$line[$i-1].';'.$line[$i].';';
			}
			elsif (length $line[$i-3] != 1 and $line[$i] > $non_mono_max -1)
			{
				$line .= $line[$i-3].';'.$line[$i-2].';'.$line[$i-1].';'.$line[$i].';';
			}		
		}
		@line = split(';', $line);
		for (my $i = 5; $i< scalar@line; $i= $i+4)
		{
			if (exists $line[$i+3])
			{
				if ($line[$i+3] - $line[$i] > $diff)
				{
					if ($beg == 0)
					{
						print OUT "$line[$i-2];$line[$i-1];$line[$i];$line[$i+1];";
					}
					else
					{
						if ($line[$i+1]>$rep and length $line[$i-2] > 1)
						{
							$rep = $line[$i+1];
						}
						print OUT "POOL;$beg;$line[$i];$rep;";
						$beg = 0;
						$end = 0;
						$rep = 0;	
					}
				}
				else
				{
					$end = $line[$i+4];
					if ($beg == 0)
					{
						$beg = $line[$i-1];	
					}
					if ($line[$i+1]>$rep and length $line[$i-2] > 1)
					{
						$rep = $line[$i+1];
					}
				}
# end if
			}
			else
			{
				if ($beg == 0)
				{
					print OUT "$line[$i-2];$line[$i-1];$line[$i];$line[$i+1];";
				}
				else
				{
					if ($line[$i+1]>$rep and length $line[$i-2] > 1)
					{
						$rep = $line[$i+1];
					}
					print OUT "POOL;$beg;$line[$i];$rep;";
					$beg = 0;
					$end = 0;
					$rep = 0;	
				}
# end else
			}
# end for
		}
		print OUT "\n";

#end while
	}
close SEQ;
close OUT;
# end if file_mot
}
#end foreach filename
}

exit;


