#!/usr/bin/perl
use Text::CSV_XS;
# use strict;

$\="\r\n";
$/="\n";
$,="\t";

die "Usage: $0 <tbl> <fasta> [<outdir>]" if (@ARGV < 2 || @ARGV > 3);

# répertoire de sortie
my $outdir;
if(@ARGV == 3) {
  $outdir = $ARGV[2];
  $outdir .= '/' if($outdir !~ /\/$/);
  mkdir $outdir if (! -d $outdir);
} else {
  $outdir = './';
}

# lecture du fasta
my %SEQHEADER;
my @SEQNUM;
my %SEQUENCES;
readFasta($ARGV[1]);


# lecture du tbl
my $csv = Text::CSV_XS->new({ binary => 1, eol => $/, sep_char => ";" });
open my $f, "<", $ARGV[0];
my $head = $csv->getline($f);
my @HEAD = @$head;
$csv->column_names(@HEAD);

# index général des séquences
open T, ">".$outdir."index.html";
print T '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2//EN">';
print T '<HTML>';
print T '<HEAD>';
print T '	<META HTTP-EQUIV="CONTENT-TYPE" CONTENT="text/html; charset=utf-8">';
print T '	<TITLE>'.$ARGV[0].'</TITLE>';
print T '</HEAD>';
print T '<BODY>';
print T '<H1>'.$ARGV[0].'</H1>';
print T '<TABLE border=1>';
print T '<TH>';
foreach $d (@HEAD) {
  print T '    <TD>'.$d.'</TD>';
}
print T '</TH>';

# itération du fichier tbl
while (my $row = $csv->getline($f)) {
  next if ($row->[0] =~ /^Total number of markers/);
  my @SEQ_CODE = split('_', $row->[0]);
  pop @SEQ_CODE;
  my $seq_root = join('_', @SEQ_CODE);
  #print STDERR $seq_root;
  # recherche de la séquence correspondante dans le fasta
  undef $seqheader;
  my $seqheader = $SEQHEADER{$seq_root};
  my $sequence = $SEQUENCES{$seq_root};
  if (!defined($seqheader)) {
    print STDERR 'ERROR : No sequence for "'.$seq_root.'"';
  } else {
    open F, ">".$outdir.$seq_root.'.fas';
    print F $seqheader;
    print F $sequence;
    close F;
    print T '  <TR>';
    print T '    <TD><A href="./'.$seq_root.'.fas">'.$seq_root.'</A></TD>';
    foreach $d (@$row) {
      print T '    <TD>'.$d.'</TD>';
    }
    print T '  </TR>';
  }
}
print T '</TABLE>';
print T '</BODY>';
print T '</HTML>';
close T;

###########################################################################################################
sub readFasta {
  my $line;
  my $first;
  my $head;
  my @HEAD;
  open(FILE, "<@_[0]") || die "Cannot open FASTA file. Stop.\n";
  $first = 0;
  while (defined($line = <FILE>))
  {
    $line =~ s/[\r\n]+//;
    if ($line =~ /^>/) {
	    if ($first == 1) {
	      $SEQHEADER{$HEAD[0]} = '>'.$head;
	    }
	    $head = $line;
	    $head =~ s/^>//;
	    @HEAD = ();
	    @HEAD = split(' ', $head);
            #print STDERR $head."--------------------------".$HEAD[0];
	    if ($first == 0) {
		    $first = 1;
	    }
	    next;
    }
    if ($first == 0) {
	    die "Not a standard FASTA file. Stop.\n";
    }
    $SEQUENCES{$HEAD[0]} .= $line;
  }
  $SEQHEADER{$HEAD[0]} = '>'.$head; # le pt'it dernier à ne pas oublier
  close(FILE);
}


